﻿using System;
using System.Windows;
using System.Windows.Controls;
using TestMapSuiteWPF.Components;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;


namespace TestShapeFile
{
   
   
    /// <summary>
    /// Interaction logic for MapControl.xaml
    /// </summary>
    public partial class MapControl : UserControl
    {
        public MapControl()
        {
            InitializeComponent();
        }


        private void wpfMap1_Loaded(object sender, RoutedEventArgs e)
        {
            wpfMap1.MapUnit = GeographyUnit.Meter;

            Proj4Projection proj4 = new Proj4Projection();
            proj4.InternalProjectionParametersString = Proj4Projection.GetEpsgParametersString(4326);
            proj4.ExternalProjectionParametersString = Proj4Projection.GetGoogleMapParametersString();


            string sMapShapeFileName = AppDomain.CurrentDomain.BaseDirectory + @"Resources\map\countries02.shp";
            string sMapShapeIndexFileName = AppDomain.CurrentDomain.BaseDirectory + @"Resources\map\countries02.idx";

            ShapeFileFeatureLayer worldLayer = new ShapeFileFeatureLayer(sMapShapeFileName, sMapShapeIndexFileName);
            worldLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Country1;
            worldLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.InternationalBorder2;
            worldLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            worldLayer.FeatureSource.Projection = proj4;

            InMemoryFeatureLayer fLayer = new InMemoryFeatureLayer();
            fLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.Capital1;
            fLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            fLayer.FeatureSource.Projection = proj4;
            PointShape point = new PointShape();
            point.X = -71.55;
            point.Y = -32.75;
            fLayer.InternalFeatures.Add(new Feature(point));

            LayerOverlay worldOverlay = new LayerOverlay();
            worldOverlay.Layers.Add("WorldLayer", worldLayer);
            worldOverlay.WrappingMode = WrappingMode.WrapDateline;
            wpfMap1.Overlays.Add(worldOverlay);

            LayerOverlay staticOverlay = new LayerOverlay();
            staticOverlay.Layers.Add("TestLayer", fLayer);
            staticOverlay.WrappingMode = WrappingMode.WrapDateline;
            wpfMap1.Overlays.Add(staticOverlay);

            string cacheDirectory = AppDomain.CurrentDomain.BaseDirectory + @"Resources\Tiles\";
            string cacheId = "mercator";

            GenerateTiles.Generate(wpfMap1.ZoomLevelSet, worldLayer, wpfMap1.MapUnit, 256, 256, cacheDirectory, cacheId);
            FileBitmapTileCache bitmapTileCache = new FileBitmapTileCache();
            bitmapTileCache.ClearCache();
            bitmapTileCache.CacheDirectory = cacheDirectory;
            bitmapTileCache.CacheId = cacheId;
            bitmapTileCache.ImageFormat = TileImageFormat.Png;

            // we need set the bounding box unit to meter here. 
            // Although, we will set this property on our side, 
            // but setting this property will restore the property to its default values,
            // so that the following bounding box won't be set properly.
            bitmapTileCache.TileMatrix.BoundingBoxUnit = GeographyUnit.Meter;

            // the wrapping date line has some limits that the max bounding box's width and height need to be n times of the real world.
            // the default max extent doesn't follow this rule. We need change it manually here as well as the method we generate the cache.
            bitmapTileCache.TileMatrix.BoundingBox = GenerateTiles.MaxExtentForWrappingDateline;
            worldOverlay.TileCache = bitmapTileCache;

            ManagedProj4Projection p = new ManagedProj4Projection();
            p.InternalProjectionParametersString = ManagedProj4Projection.GetEpsgParametersString(4326);
            p.ExternalProjectionParametersString = ManagedProj4Projection.GetGoogleMapParametersString();
            p.Open();
            worldLayer.Open();

            wpfMap1.CurrentExtent = worldLayer.GetBoundingBox();
            worldLayer.Close();
            wpfMap1.Refresh();
        }
    }
}
